from flask import Blueprint
from flask import Response
from flask import jsonify
from flask import request
from mkwvconf import Mkwvconf
from mkwvconf.mkwvconf import DEFAULT_MODEM_DEVICE
from mkwvconf.mkwvconf import DEFAULT_PROFILE_NAME

from opwen_email_client.webapp.cache import cache

blueprint = Blueprint('mkwvconf', __name__)


def create_mkwvconf() -> Mkwvconf:
    return Mkwvconf({
        option: request.args.get(option, default)
        for option, default in (
            ('modemDevice', DEFAULT_MODEM_DEVICE),
            ('profileName', DEFAULT_PROFILE_NAME),
        )
    })


@blueprint.route('/<country>/<provider>/<apn>')
@cache.memoize(timeout=600)
def get_config(country: str, provider: str, apn: str) -> Response:
    mkwvconf = create_mkwvconf()
    parameters = mkwvconf.getConfigParameters(country, provider, apn)
    config = mkwvconf.formatConfig(parameters)
    return jsonify({'config': config})


@blueprint.route('/<country>/<provider>')
@cache.memoize(timeout=600)
def list_apns(country: str, provider: str) -> Response:
    mkwvconf = create_mkwvconf()
    apns = mkwvconf.getApns(country, provider)
    return jsonify({'apns': apns})


@blueprint.route('/<country>')
@cache.memoize(timeout=600)
def list_providers(country: str) -> Response:
    mkwvconf = create_mkwvconf()
    providers = mkwvconf.getProviders(country)
    return jsonify({'providers': providers})


@blueprint.route('/')
@cache.memoize(timeout=600)
def list_countries() -> Response:
    mkwvconf = create_mkwvconf()
    countries = mkwvconf.getCountryCodes()
    return jsonify({'countries': countries})



wvdial = ('[Dialer Defaults]\n'
          'Init1 = ATZ\n'
          'Init2 = ATQ0\n'
          'Init3 = AT+CGDCONT=1,"IP","apn.konekt.io"\n'
          'Phone = *99***1#\n'
          'Stupid Mode = 1\n'
          'Username = { }\n'
          'Password = { }\n'
          'Modem Type = Analog Modem\n'
          'Modem = /dev/ttyUSB0\n'
          'IDSN = 0\n')



from datetime import datetime

from sqlalchemy import BLOB
from sqlalchemy import Boolean
from sqlalchemy import Column
from sqlalchemy import DateTime
from sqlalchemy import ForeignKey
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import Table
from sqlalchemy import Text
from sqlalchemy import exists
from sqlalchemy import or_
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy.orm import sessionmaker

from opwen_email_client.domain.email.store import EmailStore
from opwen_email_client.util.sqlalchemy import create_database
from opwen_email_client.util.sqlalchemy import get_or_create
from opwen_email_client.util.sqlalchemy import session

_Base = declarative_base()

_EmailTo = Table(
    'emailto',
    _Base.metadata,
    Column('email_id', Integer, ForeignKey('email.uid')),
    Column('to_id', Integer, ForeignKey('to.id')),
)

_EmailCc = Table(
    'emailcc',
    _Base.metadata,
    Column('email_id', Integer, ForeignKey('email.uid')),
    Column('cc_id', Integer, ForeignKey('cc.id')),
)

_EmailBcc = Table(
    'emailbcc',
    _Base.metadata,
    Column('email_id', Integer, ForeignKey('email.uid')),
    Column('bcc_id', Integer, ForeignKey('bcc.id')),
)

_EmailAttachment = Table(
    'emailattachment',
    _Base.metadata,
    Column('email_id', Integer, ForeignKey('email.uid')),
    Column('attachment_id', Integer, ForeignKey('attachment.uid')),
)


class _To(_Base):
    __tablename__ = 'to'
    id = Column(Integer, primary_key=True)

    address = Column(String(length=128), index=True, unique=True)


class _Cc(_Base):
    __tablename__ = 'cc'
    id = Column(Integer, primary_key=True)

    address = Column(String(length=128), index=True, unique=True)


class _Bcc(_Base):
    __tablename__ = 'bcc'
    id = Column(Integer, primary_key=True)

    address = Column(String(length=128), index=True, unique=True)


class _Attachment(_Base):
    __tablename__ = 'attachment'

    uid = Column(String(length=64), primary_key=True)
    filename = Column(Text)
    content = Column(BLOB)
    cid = Column(Text)

    def to_dict(self):
        return {
            '_uid': self.uid,
            'filename': self.filename,
            'content': self.content,
            'cid': self.cid,
        }

    @classmethod
    def from_dict(cls, db, attachment):
        self = db.query(_Attachment).get(attachment.get('_uid'))
        if not self:
            self = _Attachment(
                uid=attachment.get('_uid'),
                filename=attachment.get('filename'),
                content=attachment.get('content'),
                cid=attachment.get('cid'),
            )

        for pointer in attachment.pop('emails', []):
            if isinstance(pointer, _Email):
                email = pointer
                add_to_session = False
            else:
                email = db.query(_Email).get(pointer)
                add_to_session = True

            if email:
                email.attachments.append(self)
                if add_to_session:
                    db.add(email)

        return self


class _Email(_Base):
    __tablename__ = 'email'

    uid = Column(String(length=64), primary_key=True)
    subject = Column(Text)
    body = Column(Text)
    sent_at = Column(DateTime())
    read = Column(Boolean, default=False, nullable=False)
    sender = Column(String(length=128), index=True)
    attachments = relationship(_Attachment, secondary=_EmailAttachment, backref='emails', lazy='joined')
    to = relationship(_To, secondary=_EmailTo, lazy='joined')
    cc = relationship(_Cc, secondary=_EmailCc, lazy='joined')
    bcc = relationship(_Bcc, secondary=_EmailBcc, lazy='joined')

    def to_dict(self):
        attachments = self.attachments
        attachments = ([attachment.to_dict() for attachment in attachments] if attachments else None)

        sent_at = self.sent_at
        sent_at = (sent_at.strftime('%Y-%m-%d %H:%M') if sent_at else None)

        return {
            k: v
            for (k, v) in (
                ('from', self.sender),
                ('to', [_.address for _ in self.to]),
                ('cc', [_.address for _ in self.cc]),
                ('bcc', [_.address for _ in self.bcc]),
                ('subject', self.subject),
                ('body', self.body),
                ('_uid', self.uid),
                ('sent_at', sent_at),
                ('read', self.read),
                ('attachments', attachments),
            ) if v
        }

    @classmethod
    def from_dict(cls, db, email):
        sent_at = email.get('sent_at')
        sent_at = (datetime.strptime(sent_at, '%Y-%m-%d %H:%M') if sent_at else None)

        self = _Email(
            uid=email['_uid'],
            to=[get_or_create(db, _To, address=_.lower()) for _ in email.get('to', [])],
            cc=[get_or_create(db, _Cc, address=_.lower()) for _ in email.get('cc', [])],
            bcc=[get_or_create(db, _Bcc, address=_.lower()) for _ in email.get('bcc', [])],
            subject=email.get('subject'),
            body=email.get('body'),
            sent_at=sent_at,
            read=email.get('read', False),
            sender=email.get('from', '').lower() or None,
        )

        for attachment in email.get('attachments', []):
            attachment['emails'] = [self]
            db.add(_Attachment.from_dict(db, attachment))

        return self

    @classmethod
    def is_sent_by(cls, email_address):
        email_address = email_address.lower()
        return cls.sender == email_address

    @classmethod
    def is_received_by(cls, email_address):
        email_address = email_address.lower()
        to = cls.to.any(_To.address == email_address)
        cc = cls.cc.any(_Cc.address == email_address)
        bcc = cls.bcc.any(_Bcc.address == email_address)
        return to | cc | bcc


class _SqlalchemyEmailStore(EmailStore):
    def __init__(self, page_size: int, database_uri: str, restricted=None):
        super().__init__(restricted)
        self._page_size = page_size
        self._base = _Base
        self._engine = create_database(database_uri, self._base)
        self._sesion_maker = sessionmaker(autocommit=False, autoflush=False, bind=self._engine)

    def _dbread(self):
        return session(self._sesion_maker, commit=False)

    def _dbwrite(self):
        return session(self._sesion_maker, commit=True)

    def _create(self, emails_or_attachments):
        last_type = ''

        with self._dbwrite() as db:
            for email_or_attachment in emails_or_attachments:
                type_ = email_or_attachment.get('_type', '')
                if type_ != last_type:
                    db.commit()
                    last_type = type_

                if not type_ or type_ == 'email':
                    self._create_email(db, email_or_attachment)
                elif type_ == 'attachment':
                    self._create_attachment(db, email_or_attachment)

    @classmethod
    def _create_email(cls, db, email):
        uid_exists = exists().where(_Email.uid == email['_uid'])
        if not db.query(uid_exists).scalar():
            db.add(_Email.from_dict(db, email))

    @classmethod
    def _create_attachment(cls, db, attachment):
        uid_exists = exists().where(_Attachment.uid == attachment['_uid'])
        if not db.query(uid_exists).scalar():
            db.add(_Attachment.from_dict(db, attachment))

    def _mark_sent(self, uids):
        now = datetime.utcnow()
        set_sent_at = {_Email.sent_at: now}

        with self._dbwrite() as db:
            db.query(_Email)\
                .filter(_match_email_uid(uids))\
                .update(set_sent_at, synchronize_session='fetch')

    def _mark_read(self, email_address, uids):
        set_read = {_Email.read: True}

        with self._dbwrite() as db:
            db.query(_Email)\
                .filter(_match_email_uid(uids) & _can_access(email_address))\
                .update(set_read, synchronize_session='fetch')

    def _delete(self, email_address, uids):
        should_delete = _match_email_uid(uids) & _can_access(email_address)

        with self._dbwrite() as db:
            for email in db.query(_Email).filter(should_delete).all():
                email.attachments = []
                db.delete(email)

        self._delete_orphaned_attachments()

    def _delete_orphaned_attachments(self):
        with self._dbwrite() as db:
            db.query(_Attachment)\
                .filter(~_Attachment.emails.any())\
                .delete(synchronize_session='fetch')

    def _find(self, query, table=_Email):
        with self._dbread() as db:
            results = db.query(table).filter(query)
            result = results.first()
        return result.to_dict() if result else None

    def _query(self, query, page):
        with self._dbread() as db:
            results = db.query(_Email).filter(query)
            results = results.order_by(_Email.sent_at.desc())

            if page is None:
                results = results.all()
                emails = (email.to_dict() for email in results)
            else:
                page = max(0, page - 1)
                results = results.offset(page * self._page_size).limit(self._page_size)
                emails = [email.to_dict() for email in results]

        return emails

    def inbox(self, email_address, page):
        return self._query(_Email.is_received_by(email_address), page)

    def outbox(self, email_address, page):
        return self._query(_Email.is_sent_by(email_address) & _Email.sent_at.is_(None), page)

    def search(self, email_address, page, query):
        textquery = '%{}%'.format(query)
        contains_query = or_(*(
            _Email.subject.ilike(textquery),
            _Email.body.ilike(textquery),
            _Email.sender.ilike(textquery),
            _Email.to.any(_To.address.ilike(textquery)),
            _Email.cc.any(_Cc.address.ilike(textquery)),
            _Email.bcc.any(_Bcc.address.ilike(textquery)),
        ))
        return self._query(_can_access(email_address) & contains_query, page)

    def pending(self, page):
        return self._query(_Email.sent_at.is_(None), page)

    def has_unread(self, email_address):
        with self._dbread() as db:
            unread = exists().where(_Email.read.is_(False) & _Email.is_received_by(email_address))
            has_unread_emails = db.query(unread).scalar()
        return has_unread_emails

    def num_pending(self):
        with self._dbread() as db:
            count = db.query(_Email)\
                .filter(_Email.sent_at.is_(None))\
                .count()
        return count

    def get(self, uid):
        return self._find(_Email.uid == uid)

    def get_attachment(self, email_id, attachment_id):
        return self._find(_Attachment.uid == attachment_id, table=_Attachment)

    def sent(self, email_address, page):
        return self._query(_Email.is_sent_by(email_address) & _Email.sent_at.isnot(None), page)


class SqliteEmailStore(_SqlalchemyEmailStore):
    def __init__(self, page_size: int, database_path: str, restricted=None):
        super().__init__(
            page_size=page_size,
            database_uri='sqlite:///{}'.format(database_path),
            restricted=restricted,
        )


def _can_access(email_address):
    return _Email.is_sent_by(email_address) | _Email.is_received_by(email_address)


def _match_email_uid(uids):
    return _Email.uid.in_(uids)



from os import path
from tempfile import gettempdir

from babel import Locale
from environs import Env
from flask_babelex import gettext as _

from opwen_email_client.util.os import subdirectories

env = Env()
settings_path = env('OPWEN_SETTINGS', None)
env.read_env(settings_path, recurse=False)

app_basedir = path.abspath(path.dirname(__file__))
root_domain = env('OPWEN_ROOT_DOMAIN', 'lokole.ca')


# noinspection PyPep8Naming
class ImageDimensions(object):
    MAX_WIDTH_IMAGES = env.int('LOKOLE_MAX_WIDTH_EMAIL_IMAGES', 200)
    MAX_HEIGHT_IMAGES = env.int('LOKOLE_MAX_HEIGHT_EMAIL_IMAGES', 200)


# noinspection PyPep8Naming
class i8n(object):
    SETTINGS_UPDATED = _('Settings updated! If required, the app will restart soon to reflect the changes.')
    LOGIN_REQUIRED = _('Please log in to access this page.')
    UNAUTHORIZED = _('You do not have permission to view this page.')
    INVALID_PASSWORD = _('Invalid password.')
    SHORT_PASSWORD = _('The password must have at least 6 characters.')
    EMAIL_SENT = _('Email sent!')
    EMAIL_ADDRESS_INVALID = _('Invalid email address.')
    EMAIL_TO_REQUIRED = _('Please specify a recipient.')
    EMAIL_NO_SUBJECT = _('(no subject)')
    LOGGED_IN = _('You are now logged in.')
    LOGGED_OUT = _('You have logged out successfully.')
    WELCOME = _('Welcome!')
    EMAIL_CHARACTERS = _('Please only use letters, numbers, dots and dashes.')
    FORBIDDEN_ACCOUNT = _('This account name is not available.')
    ACCOUNT_CREATED = _('Your Lokole account has been created successfully!')
    ACCOUNT_SUSPENDED = _('Your account has been suspended. Please contact your administrator.')
    SYNC_RUNNING = _('Email synchronization running and will complete soon.')
    UPDATE_RUNNING = _('Code update is now running. The app will restart soon to reflect the updates.')
    SYNC_SCHEDULE_SYNTAX_DESCRIPTION = _('The syntax is: "minute hour day-of-month month day-of-week". '
                                         'Use "*" for any value or "," to separate multiple values '
                                         'or "-" to define a range of values or "/" for step values.')
    UNEXPECTED_ERROR = _('Unexpected error. Please contact your administrator.')
    PAGE_DOES_NOT_EXIST = _('This page does not exist.')
    USER_DOES_NOT_EXIST = _('This user does not exist.')
    USER_SUSPENDED = _('The user was suspended.')
    USER_UNSUSPENDED = _('The user was un-suspended.')
    USER_PROMOTED = _('The user now is an administrator.')
    ALREADY_PROMOTED = _('The user already is an administrator.')
    ADMIN_CANNOT_BE_SUSPENDED = _("Administrators can't be suspended.")
    ADMIN_PASSWORD_CANNOT_BE_RESET = _("Administrator password can't be " "reset.")
    PASSWORD_CHANGED_BY_ADMIN = _('Password was reset by administrator to: ')
    SAME_PASSWORD = _(' Your new password must be different than your previous password.')


class AppConfig(object):
    CACHE_TYPE = 'simple'
    STATE_BASEDIR = path.abspath(env('OPWEN_STATE_DIRECTORY', gettempdir()))
    SQLITE_PATH = path.join(STATE_BASEDIR, 'users.sqlite3')
    SQLALCHEMY_DATABASE_URI = 'sqlite:///' + SQLITE_PATH
    SQLALCHEMY_MIGRATE_REPO = path.join(STATE_BASEDIR, 'app.migrate')
    SQLALCHEMY_TRACK_MODIFICATIONS = False

    SECRET_KEY = env('OPWEN_SESSION_KEY', None)

    CELERY_SQLITE_PATH = path.join(STATE_BASEDIR, 'celery.sqlite3')
    CELERY_BROKER_URL = env('CELERY_BROKER_URL', 'sqlalchemy+sqlite:///' + CELERY_SQLITE_PATH)
    CELERY_BEAT_SCHEDULE_FILENAME = path.join(STATE_BASEDIR, 'celery.cron')

    SECURITY_USER_IDENTITY_ATTRIBUTES = 'email'
    SECURITY_PASSWORD_HASH = 'bcrypt'  # nosec
    SECURITY_PASSWORD_SINGLE_HASH = True
    SECURITY_REGISTERABLE = env.bool('OPWEN_CAN_REGISTER_USER', True)
    SECURITY_CHANGEABLE = env.bool('OPWEN_CAN_CHANGE_PASSWORD', True)
    SECURITY_TRACKABLE = True
    SECURITY_SEND_PASSWORD_CHANGE_EMAIL = False
    SECURITY_POST_REGISTER_VIEW = 'register_complete'
    SECURITY_POST_LOGIN_VIEW = 'login_complete'
    SECURITY_POST_LOGOUT_VIEW = 'logout_complete'
    SECURITY_SEND_REGISTER_EMAIL = False
    SECURITY_MSG_LOGIN = i8n.LOGIN_REQUIRED, 'error'
    SECURITY_MSG_UNAUTHORIZED = i8n.UNAUTHORIZED, 'error'
    SECURITY_MSG_INVALID_PASSWORD = i8n.INVALID_PASSWORD, 'error'
    SECURITY_MSG_DISABLED_ACCOUNT = i8n.ACCOUNT_SUSPENDED, 'error'
    SECURITY_MSG_PASSWORD_INVALID_LENGTH = i8n.SHORT_PASSWORD, 'error'
    SECURITY_MSG_PASSWORD_IS_THE_SAME = i8n.SAME_PASSWORD, 'error'
    SECURITY_LOGIN_URL = '/user/login'
    SECURITY_LOGOUT_URL = '/user/logout'
    SECURITY_REGISTER_URL = '/user/register'
    SECURITY_CHANGE_URL = '/user/password/change'

    TESTING = env.bool('OPWEN_ENABLE_DEBUG', False)

    MODEM_CONFIG_DIR = path.join(STATE_BASEDIR, 'usb_modeswitch')
    SIM_CONFIG_DIR = path.join(STATE_BASEDIR, 'wvdial')
    LOCAL_EMAIL_STORE = path.join(STATE_BASEDIR, 'emails.sqlite3')
    SIM_TYPE = env('OPWEN_SIM_TYPE', None)
    RESTART_PATHS = env.dict('OPWEN_RESTART_PATH', {})
    MAX_UPLOAD_SIZE_MB = env.int('OPWEN_MAX_UPLOAD_SIZE_MB', 0)

    SYNC_SCHEDULE = env('OPWEN_SYNC_SCHEDULE', '').strip()

    EMAIL_ADDRESS_DELIMITER = ','
    EMAILS_PER_PAGE = env.int('OPWEN_EMAILS_PER_PAGE', 10)

    LOCALES_DIRECTORY = path.join(app_basedir, 'translations')
    DEFAULT_LOCALE = Locale.parse('en_ca')
    LOCALES = [DEFAULT_LOCALE] + [Locale.parse(code) for code in subdirectories(LOCALES_DIRECTORY)]

    EMAIL_SEARCHABLE = env.bool('OPWEN_CAN_SEARCH_EMAIL', True)
    COMPRESSION = env('OPWEN_COMPRESSION', 'zstd')
    EMAIL_SERVER_HOSTNAME = env('OPWEN_EMAIL_SERVER_HOSTNAME', None)
    EMAIL_HOST_FORMAT = '{}.' + root_domain
    STORAGE_PROVIDER = env('LOKOLE_STORAGE_PROVIDER', 'AZURE_BLOBS')
    STORAGE_CONTAINER = env('OPWEN_REMOTE_RESOURCE_CONTAINER', None)
    STORAGE_ACCOUNT_NAME = env('OPWEN_REMOTE_ACCOUNT_NAME', None)
    STORAGE_ACCOUNT_KEY = env('OPWEN_REMOTE_ACCOUNT_KEY', None)
    CLIENT_NAME = env('OPWEN_CLIENT_NAME', None)
    CLIENT_ID = env('OPWEN_CLIENT_ID', None)
    CLIENT_EMAIL_HOST = EMAIL_HOST_FORMAT.format(CLIENT_NAME)
    NEWS_INBOX = 'news@{}'.format(CLIENT_EMAIL_HOST)
    ADMIN_INBOX = 'admin@{}'.format(CLIENT_EMAIL_HOST)
    NEWS_SENDERS = set(env.list('OPWEN_NEWS_SENDERS', []))
    FORBIDDEN_ACCOUNTS = [NEWS_INBOX, ADMIN_INBOX]

    IOC = env('LOKOLE_IOC', 'opwen_email_client.webapp.ioc.Ioc')

    APP_ROOT = env('OPWEN_APP_ROOT', '').rstrip('/')
    SECURITY_URL_PREFIX = APP_ROOT or None



from contextlib import contextmanager

from sqlalchemy import create_engine
from sqlalchemy.exc import IntegrityError
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import scoped_session
from sqlalchemy.orm.exc import NoResultFound


def create_database(uri: str, base):
    engine = create_engine(uri)

    try:
        base.metadata.create_all(bind=engine)
    except SQLAlchemyError:
        pass

    return engine


def get_or_create(db, model, **model_args):
    try:
        return db.query(model).filter_by(**model_args).one()
    except NoResultFound:
        pass

    created = model(**model_args)
    try:
        db.add(created)
        db.flush()
        return created
    except IntegrityError:
        pass

    db.rollback()
    return db.query(model).filter_by(**model_args).one()


@contextmanager
def session(session_maker, commit: bool = False):
    session_factory = scoped_session(session_maker)
    db = session_factory()

    try:
        yield db
        if commit:
            db.commit()
    except SQLAlchemyError:
        db.rollback()
    finally:
        db.close()



from os import environ
from pathlib import Path
from typing import Optional

from flask_wtf import FlaskForm
from wtforms import StringField
from wtforms import SubmitField
from wtforms import TextAreaField

from opwen_email_client import __version__
from opwen_email_client.util.os import replace_line
from opwen_email_client.util.wtforms import CronSchedule
from opwen_email_client.webapp.actions import RestartApp
from opwen_email_client.webapp.config import AppConfig
from opwen_email_client.webapp.config import i8n
from opwen_email_client.webapp.config import settings_path
from opwen_email_client.webapp.tasks import update


class SettingsForm(FlaskForm):
    wvdial = TextAreaField()

    code_version = StringField()

    sim_type = StringField()

    sync_schedule = StringField(validators=[CronSchedule()], description=i8n.SYNC_SCHEDULE_SYNTAX_DESCRIPTION)

    submit = SubmitField()

    def update(self):
        restart_required = False
        restart_required |= self._update_sim_type()
        restart_required |= self._update_sync_schedule()
        restart_required |= self._update_wvdial()
        restart_required |= self._update_code_version()

        if restart_required:
            restart_app = RestartApp(restart_paths=AppConfig.RESTART_PATHS)
            restart_app()

    def _update_wvdial(self) -> bool:
        wvdial = self.wvdial.data.strip()
        path = _get_wvdial_path(self.sim_type.data.strip())
        if not wvdial or wvdial == _read_wvdial(path):
            return False

        path.parent.mkdir(parents=True, exist_ok=True)

        with path.open('w', encoding='utf-8') as fobj:
            fobj.write('\n'.join(line.strip() for line in wvdial.splitlines()))
        return False

    def _update_code_version(self) -> bool:
        code_version = self.code_version.data.strip()
        if not code_version or code_version == __version__:
            return False

        update.delay(version=code_version)
        return False

    def _update_sim_type(self) -> bool:
        sim_type = self.sim_type.data.strip()
        if not sim_type or sim_type == AppConfig.SIM_TYPE:
            return False

        self._update_config('OPWEN_SIM_TYPE', sim_type)
        return True

    def _update_sync_schedule(self) -> bool:
        sync_schedule = self.sync_schedule.data.strip()
        if sync_schedule == AppConfig.SYNC_SCHEDULE:
            return False

        self._update_config('OPWEN_SYNC_SCHEDULE', sync_schedule)
        return True

    @classmethod
    def _update_config(cls, env_key: str, value: str):
        def is_env(line):
            return line.startswith('{}='.format(env_key))

        replace_line(settings_path, is_env, '{}={}'.format(env_key, value))

        config_key = env_key.replace('OPWEN_', '')
        setattr(AppConfig, config_key, value)
        environ[env_key] = value

        return True

    @classmethod
    def from_config(cls):
        return cls(
            code_version=__version__,
            wvdial=_read_wvdial(_get_wvdial_path()),
            sync_schedule=AppConfig.SYNC_SCHEDULE,
            sim_type=AppConfig.SIM_TYPE,
        )


def _get_wvdial_path(sim_type: Optional[str] = AppConfig.SIM_TYPE) -> Path:
    return Path(AppConfig.SIM_CONFIG_DIR) / sim_type


def _read_wvdial(path: Path) -> str:
    if not path.is_file():
        return ''

    with path.open(encoding='utf-8') as fobj:
        return fobj.read().strip()



from logging import getLogger

from opwen_email_client.webapp.ioc import create_app

app = create_app()

if __name__ != '__main__':
    gunicorn_logger = getLogger('gunicorn.error')
    app.logger.handlers = gunicorn_logger.handlers
    app.logger.setLevel(gunicorn_logger.level)

from opwen_email_client.webapp import jinja  # noqa: F401,E402  # isort:skip
from opwen_email_client.webapp import login  # noqa: F401,E402  # isort:skip
from opwen_email_client.webapp import views  # noqa: F401,E402  # isort:skip
